"""
`stemma_soil`
================================================================================

Python library for Hydros21 or Decagon CDT-10 water level sensor.

**Author(s):** `Colby Sawyer <https://github.com/ColbySawyer7>`_

Implementation Notes
--------------------
**Hardware:**
* `Stemma Soil Sensor `_

**Software and Dependencies:**

"""

import time
import board
from adafruit_seesaw.seesaw import Seesaw

def get_data():
    """_summary_

    Returns:
        [int]: _description_
    """
    i2c_bus = board.I2C()
    ss = Seesaw(i2c_bus, addr=0x36)
    # read moisture level through capacitive touch pad
    touch = ss.moisture_read()
    # read temperature from the temperature sensor
    temp = ss.get_temp()

    print("temp: " + str(temp) + "  moisture: " + str(touch))
    return [temp, touch]



"""
`pm25`
================================================================================

Python library for PM25 Air Quality Sensor.

**Author(s):** `Colby Sawyer <https://github.com/ColbySawyer7>`_

Implementation Notes
--------------------
**Hardware:**
* ``_

**Software and Dependencies:**

"""

# pylint: disable=unused-import
import time
import serial
import board
import busio
from digitalio import DigitalInOut, Direction, Pull
from adafruit_pm25.i2c import PM25_I2C

class PM25:
    pm10_std= 0
    pm25_std= 0
    pm100_std= 0
    pm10_env= 0
    pm25_env= 0
    pm100_env= 0
    part_03um= 0
    part_05um= 0
    part_10um= 0
    part_25um= 0
    part_50um= 0
    part_100um= 0

    def __init__(self,pm10_std= 0,pm25_std= 0,pm100_std= 0,pm10_env= 0,pm25_env= 0,pm100_env= 0,part_03um= 0,part_05um= 0,part_10um= 0,part_25um= 0,part_50um= 0,part_100um= 0):
    
        self.pm10_std= 0
        self.pm25_std= 0
        self.pm100_std= 0
        self.pm10_env= 0
        self.pm25_env= 0
        self.pm100_env= 0
        self.part_03um= 0
        self.part_05um= 0
        self.part_10um= 0
        self.part_25um= 0
        self.part_50um= 0
        self.part_100um= 0

    def get_date(self, sample_size=10,dec_factor=100):

        reset_pin = None
        # If you have a GPIO, its not a bad idea to connect it to the RESET pin

        # reset_pin = DigitalInOut(board.G0)
        # reset_pin.direction = Direction.OUTPUT
        # reset_pin.value = False

        # For use with Raspberry Pi/Linux:
        uart = serial.Serial("/dev/ttyS0", baudrate=9600, timeout=0.25)


        # Connect to a PM2.5 sensor over UART
        # from adafruit_pm25.uart import PM25_UART
        # pm25 = PM25_UART(uart, reset_pin)

        # Create library object, use 'slow' 100KHz frequency!
        i2c = busio.I2C(board.SCL, board.SDA, frequency=100000)
        # Connect to a PM2.5 sensor over I2C
        pm25 = PM25_I2C(i2c, reset_pin)

        print("Found PM2.5 sensor, reading data...")

        try:
            aqdata = pm25.read()
            print("Concentration Units (standard)")
            print("---------------------------------------")
            print(
                "PM 1.0: %d\tPM2.5: %d\tPM10: %d"
                % (aqdata["pm10 standard"], aqdata["pm25 standard"], aqdata["pm100 standard"])
            )
            print("Concentration Units (environmental)")
            print("---------------------------------------")
            print(
                "PM 1.0: %d\tPM2.5: %d\tPM10: %d"
                % (aqdata["pm10 env"], aqdata["pm25 env"], aqdata["pm100 env"])
            )
            print("---------------------------------------")
            print("Particles > 0.3um / 0.1L air:", aqdata["particles 03um"])
            print("Particles > 0.5um / 0.1L air:", aqdata["particles 05um"])
            print("Particles > 1.0um / 0.1L air:", aqdata["particles 10um"])
            print("Particles > 2.5um / 0.1L air:", aqdata["particles 25um"])
            print("Particles > 5.0um / 0.1L air:", aqdata["particles 50um"])
            print("Particles > 10 um / 0.1L air:", aqdata["particles 100um"])

            self.pm10_std= aqdata["pm10 standard"]
            self.pm25_std= aqdata["pm25 standard"]
            self.pm100_std= aqdata["pm100 standard"]
            self.pm10_env= aqdata["pm10 env"]
            self.pm25_env= aqdata["pm25 env"]
            self.pm100_env= aqdata["pm100 env"]
            self.part_03um= aqdata["particles 03um"]
            self.part_05um= aqdata["particles 05um"]
            self.part_10um= aqdata["particles 10um"]
            self.part_25um= aqdata["particles 25um"]
            self.part_50um= aqdata["particles 50um"]
            self.part_100um= aqdata["particles 100um"]


            pm10_std = int((self.pm10_std*dec_factor))
            print("pm10 standard: %0.1f %%" % pm10_std)

            pm25_std = int((self.pm25_std*dec_factor))
            print("pm25 standard: %0.1f %%" % pm25_std)

            pm100_std = int((self.pm100_std*dec_factor))
            print("pm100 standard: %0.1f %%" % pm100_std)

            pm10_env = int((self.pm10_env*dec_factor))
            print("pm10 env: %0.1f %%" % pm10_env)

            pm25_env = int((self.pm25_env*dec_factor))
            print("pm25 env: %0.1f %%" % pm25_env)

            pm100_env = int((self.pm100_env*dec_factor))
            print("pm100 env: %0.1f %%" % pm100_env)

            part_03um = int((self.part_03um*dec_factor))
            print("Particles > 0.3um / 0.1L air:: %0.1f %%" % part_03um)

            part_05um = int((self.part_05um*dec_factor))
            print("Particles > 0.5um / 0.1L air:: %0.1f %%" % part_05um)

            part_10um = int((self.part_10um*dec_factor))
            print("Particles > 1.0um / 0.1L air:: %0.1f %%" % part_10um)

            part_25um = int((self.part_25um*dec_factor))
            print("Particles > 2.5um / 0.1L air:: %0.1f %%" % part_25um)

            part_50um = int((self.part_50um*dec_factor))
            print("Particles > 5.0um / 0.1L air:: %0.1f %%" % part_50um)

            part_100um = int((self.part_100um*dec_factor))
            print("Particles > 10.0um / 0.1L air:: %0.1f %%" % part_100um)
 
            #print(aqdata)
            sensor_data = bytearray(24)
            FEATHER_ID = 1
            sensor_data[0] = FEATHER_ID

            sensor_data[1] = (pm10_std >> 8) & 0xff
            sensor_data[2]= pm10_std & 0xff
            
            sensor_data[3] = (pm25_std >> 8) & 0xff
            sensor_data[4] = pm25_std & 0xff
            
            sensor_data[5] = (pm100_std >> 8) & 0xff
            sensor_data[6] = pm100_std & 0xff

            sensor_data[7] = (pm10_env >> 8) & 0xff
            sensor_data[8] = pm10_env & 0xff

            sensor_data[9] = (pm25_env >> 8) & 0xff
            sensor_data[10] = pm25_env & 0xff

            sensor_data[11] = (pm100_env >> 8) & 0xff
            sensor_data[12] = pm100_env & 0xff

            sensor_data[13] = (part_03um >> 8) & 0xff
            sensor_data[14] = part_03um & 0xff

            sensor_data[15] = (part_05um >> 8) & 0xff
            sensor_data[16] = part_05um & 0xff

            sensor_data[17] = (part_10um >> 8) & 0xff
            sensor_data[18] = part_10um & 0xff

            sensor_data[19] = (part_25um >> 8) & 0xff
            sensor_data[20] = part_25um & 0xff

            sensor_data[21] = (part_50um >> 8) & 0xff
            sensor_data[22] = part_50um & 0xff

            sensor_data[23] = (part_100um >> 8) & 0xff
            sensor_data[24] = part_100um & 0xff

        except: 
            sensor_data =[0]

        return sensor_data


"""
`sen5x`
================================================================================

*Python library for Sensirion SEN5xAir Quality Sensor.*

**Author(s):** `Colby Sawyer <https://github.com/ColbySawyer7>`_

Implementation Notes
--------------------
**Hardware:**
`Sensirion SEN-5x <https://sensirion.com/products/catalog/SEN54/>`_

**Software and Dependencies:**
Utilizing Sensirion provided libraries. `More Information <https://sensirion.github.io/python-i2c-driver/>`_

"""


import time
from sensirion_i2c_driver import I2cConnection, LinuxI2cTransceiver
from sensirion_i2c_sen5x import Sen5xI2cDevice

def sensor_run(sample_size):
    """Runs the sensor specific operations and collects/summarizes the data.

    Args:
        sample_size (int, mandatory): [Sample size (seconds) of collection]

    Returns:
        [int]: [Returns array of integers mapped to the corresponding values ([avg_pm1, avg_pm25,avg_pm10, temperature, humidity, laser_status])]
    """
    with LinuxI2cTransceiver('/dev/i2c-1') as i2c_transceiver:
        device = Sen5xI2cDevice(I2cConnection(i2c_transceiver))

        # Print some device information
        print("Version: {}".format(device.get_version()))
        print("Product Name: {}".format(device.get_product_name()))
        print("Serial Number: {}".format(device.get_serial_number()))

        # Perform a device reset (reboot firmware)
        device.device_reset()

        # Start measurement
        device.start_measurement()
        # Wait until next result is available
        print("Waiting for new data...")
        while device.read_data_ready() is False:
            time.sleep(0.1)

        # Read measured values -> clears the "data ready" flag
        values = device.read_measured_values()
        print(values)

        # Access a specific value separately (see Sen5xMeasuredValues)
        mass_concentration = values.mass_concentration_2p5.physical
        ambient_temperature = values.ambient_temperature.degrees_celsius

        mc_1p0 = values.mass_concentration_1p0.physical
        mc_2p5 = values.mass_concentration_2p5.physical
        mc_4p0 = values.mass_concentration_4p0.physical
        mc_10p0 = values.mass_concentration_10p0.physical
        ambient_rh = values.ambient_humidity.percent_rh
        ambient_t = values.ambient_temperature.degrees_celsius
        voc_index = values.voc_index.scaled
        nox_index = values.nox_index.scaled

        # Read device status
        status = device.read_device_status()
        print("Device Status: {}\n".format(status))

        # Stop measurement
        device.stop_measurement()
        print("Measurement stopped.")
        
        return [mc_1p0, mc_2p5,mc_4p0, mc_10p0, ambient_rh, ambient_t, voc_index, nox_index]

class SEN5x:
    """Driver class for OPC particulate sensors
    """
    mc_1p0= 0
    mc_2p5= 0
    mc_4p0= 0
    mc_10p0= 0
    ambient_rh= 0
    ambient_t= 0
    voc_index= 0
    nox_index= 0


    def __init__(self,mc_1p0= 0,mc_2p5= 0,mc_4p0= 0,mc_10p0= 0,ambient_rh= 0,ambient_t= 0,voc_index= 0,nox_index= 0):
        """_summary_

        Args:
            mc_1p0 (int, optional): _description_. Defaults to 0.
            mc_2p5 (int, optional): _description_. Defaults to 0.
            mc_4p0 (int, optional): _description_. Defaults to 0.
            mc_10p0 (int, optional): _description_. Defaults to 0.
            ambient_rh (int, optional): _description_. Defaults to 0.
            ambient_t (int, optional): _description_. Defaults to 0.
            voc_index (int, optional): _description_. Defaults to 0.
            nox_index (int, optional): _description_. Defaults to 0.
        """
    
        self.mc_1p0= mc_1p0
        self.mc_2p5= mc_2p5
        self.mc_4p0= mc_4p0
        self.mc_10p0= mc_10p0
        self.ambient_rh= ambient_rh
        self.ambient_t= ambient_t
        self.voc_index= voc_index
        self.nox_index= nox_index

    def get_data(self, sample_size=10, dec_factor=100):
        """_summary_

        Args:
            sample_size (int, optional): _description_. Defaults to 10.
            dec_factor (int, optional): _description_. Defaults to 100.
        """
        try:
            sensor_readings = sensor_run(sample_size)
            #[mc_1p0, mc_2p5,mc_4p0, mc_10p0, ambient_rh, ambient_t, voc_index, nox_index]

            self.mc_1p0= sensor_readings[0]
            self.mc_2p5= sensor_readings[1]
            self.mc_4p0= sensor_readings[2]
            self.mc_10p0= sensor_readings[3]
            self.ambient_rh= sensor_readings[4]
            self.ambient_t= sensor_readings[5]
            self.voc_index= sensor_readings[6]
            self.nox_index= sensor_readings[7]


            mc_1p0 = int((self.mc_1p0*dec_factor))
            print("mc_1p0: %0.1f %%" % mc_1p0)

            mc_2p5 = int((self.mc_2p5*dec_factor))
            print("mc_2p5: %0.1f %%" % mc_2p5)

            mc_4p0 = int((self.mc_4p0*dec_factor))
            print("mc_4p0: %0.1f %%" % mc_4p0)

            mc_10p0 = int((self.mc_10p0*dec_factor))
            print("mc_10p0: %0.1f %%" % mc_10p0)

            ambient_rh = int((self.ambient_rh*dec_factor))
            print("ambient_rh: %0.1f %%" % ambient_rh)

            ambient_t = int((self.ambient_t*dec_factor))
            print("ambient_t: %0.1f %%" % ambient_t)

            voc_index = int((self.voc_index*dec_factor))
            print("voc_index: %0.1f %%" % voc_index)

            nox_index = int((self.nox_index*dec_factor))
            print("nox_index: %0.1f %%" % nox_index)

            sensor_data = bytearray(16)
            FEATHER_ID = 1
            sensor_data[0] = FEATHER_ID

            sensor_data[1] = (mc_1p0 >> 8) & 0xff
            sensor_data[2]= mc_1p0 & 0xff
            
            sensor_data[3] = (mc_2p5 >> 8) & 0xff
            sensor_data[4] = mc_2p5 & 0xff
            
            sensor_data[5] = (mc_4p0 >> 8) & 0xff
            sensor_data[6] = mc_4p0 & 0xff

            sensor_data[7] = (mc_10p0 >> 8) & 0xff
            sensor_data[8] = mc_10p0 & 0xff

            sensor_data[9] = (ambient_rh >> 8) & 0xff
            sensor_data[10] = ambient_rh & 0xff

            sensor_data[11] = (ambient_t >> 8) & 0xff
            sensor_data[12] = ambient_t & 0xff

            sensor_data[13] = (voc_index >> 8) & 0xff
            sensor_data[14] = voc_index & 0xff

            sensor_data[15] = (nox_index >> 8) & 0xff
            sensor_data[16] = nox_index & 0xff

        except:
            sensor_data[0]



"""
`hydros`
================================================================================

*Python library for Hydros21 or Decagon CDT-10 water level sensor.*

**Author(s):** `Colby Sawyer <https://github.com/ColbySawyer7>`_

Implementation Notes
--------------------
**Hardware:**
Hydros 21 Conductivity, Temperature, and Depth Sensor `More Information <https://www.metergroup.com/en/meter-environment/products/hydros-21-water-level-sensor-conductivity-temperature-depth>`_

**Software and Dependencies:**

"""
#import serial.tools.list_ports
import serial
import time
import re

def parse_reading(val):
    """Takes string of values from sensor and parses them into String list

    Args:
        val (String): Unparsed string containing multiple reading from sensor ex. "0+40+24.4-140"

    Returns:
        [String]: Parsed list of strings each corresponding to a returned measurement ex. ["0","+40","+24.4","-140"]
    """
    res = re.findall(r'[-+][0-9.]+|\D', val)
    return res

class Hydros:
    """Driver for Hydros 12 or Decagon CDT-10 water level sensors
    """
    water_depth = 0
    temperature = 0
    electrical_conductivity = 0

    def __init__(self, water_depth=0, temperature=0, conductivity=0):
        """Initialize instance of sensor

        Args:
            water_depth (int, optional): Water Depth (mm). Defaults to 0.
            temperature (int, optional): Temperature (celsius). Defaults to 0.
            conductivity (int, optional): Conductivity (uS/cm). Defaults to 0.
        """
        self.water_depth = water_depth
        self.temperature = temperature
        self.electrical_conductivity = conductivity



    def get_data(self, dec_factor=100):
        """Collects data from the device attached to the SDI-12 USB external board

        Args:
            dec_factor (int, optional): Holds the decimal factor to be used for integer conversion] Defaults to 100.

        Raises:
            ValueError: Throws error if device is not found
            ValueError: Throws error if device readings are not properly formatted after parsing (invalid readings)

        Returns:
            [Bytearray]: Packaged up data to be sent via LoRa driving code
        """
        version = '1.0'

        print('+-' * 40)
        print('Simple SDI-12 Sensor Reader', version)
        print('+-' * 40)

        port_names=[]
        ports = serial.tools.list_ports.comports()
        user_port_selection=0
        i=0
        
        ser=serial.Serial(port=ports[int(user_port_selection)].device,baudrate=9600,timeout=10)
        time.sleep(2.5) # delay for arduino bootloader and the 1 second delay of the adapter.

        print('Connecting to sensor ...')
        
        ser.write(b'?!')
        sdi_12_line=ser.readline()
        sdi_12_line=sdi_12_line[:-2] # remove \r and \n since [0-9]$ has trouble with \r
        m=re.search(b'[0-9a-zA-Z]$',sdi_12_line) # having trouble with the \r
        #TODO CHECK that devices exsits
        if m :
            sdi_12_address=m.group(0) # find address
            print('\nSensor address:', sdi_12_address.decode('utf-8'))

            ser.write(sdi_12_address+b'I!')
            sdi_12_line=ser.readline()
            sdi_12_line=sdi_12_line[:-2] # remove \r and \n
            print('Sensor info:',sdi_12_line.decode('utf-8'))

            ser.write(sdi_12_address+b'M!')
            sdi_12_line=ser.readline()
            sdi_12_line=ser.readline()
            ser.write(sdi_12_address+b'D0!')
            sdi_12_line=ser.readline()
            sdi_12_line=sdi_12_line[:-2] # remove \r and \n

            value = sdi_12_line.decode('utf-8')

            parsed_values = parse_reading(value)
            if len(parsed_values) >= 3:
                self.depth = float(parsed_values[0])
                self.temperature = float(parsed_values[1])
                self.electrical_conductivity = float(parsed_values[2])
            else:
                self.depth = 0
                self.temperature = 0
                self.electrical_conductivity= 0
                raise ValueError('SENSOR ERROR: Reading from sensor doesnt match explicitly defined format.')

            print('Sensor reading:', parsed_values)
            
            sensor_data = bytearray(7)
            FEATHER_ID = 1

            depth_val = int((self.water_depth*dec_factor))
            print("Water Depth: %0.1f %%" % depth_val)

            temp_val = int((self.temperature*dec_factor))
            print("Temperature: %0.1f %%" % temp_val)

            conduc_val = int((self.electrical_conductivity*dec_factor))
            print("Conductivity: %0.1f %%" % conduc_val)

            sensor_data[0] = FEATHER_ID
            # Water Depth
            sensor_data[1] = (depth_val >> 8) & 0xff
            sensor_data[2]= depth_val & 0xff
            # Temperature
            sensor_data[3] = (temp_val >> 8) & 0xff
            sensor_data[4] = temp_val & 0xff
            #Conductivity
            sensor_data[5] = (conduc_val >> 8) & 0xff
            sensor_data[6] = conduc_val & 0xff

            return sensor_data
        else:
            raise ValueError('ERROR: No sensor detected')




from setuptools import setup

setup(
  name = 'esdn_sensing',         # How you named your package folder (MyLib)
  packages = ['esdn_sensing'],   # Chose the same as "name"
  version = '0.1.2',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Python library to deliver sensor specific support to ESDN infrastructure',   # Give a short description about your library
  author = 'Colby Sawyer',                   # Type in your name
  author_email = 'sawyerco21@ecu.edu',      # Type in your E-Mail
  url = 'https://github.com/ECU-Sensing/esdn_sensing',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/ECU-Sensing/esdn_sensing/archive/refs/tags/v_0.1.2.tar.gz',    # I explain this later on
  keywords = ['sensor', 'hydros', 'cdt-10', 'water', 'generic','shim','LoRa','sensors','pm2', 'opc', 'sen', 'sen54', 'sen53', 'air', 'quality'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
          'pyserial',
          'py-opc-ng',
          'pyusbiss',
          'sensirion_i2c_driver',
          'sensirion_i2c_sen5x',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
#  packages=find_packages(exclude=('tests')),
)